package eu.clarin.cmdi.mdservice.proxy;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;

import org.apache.log4j.Logger;

import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.Repositories;

/**
 * Proxy to the Metadata Repository.  
 * Responds to requests (collections, model, recordset) 
 * by dispatching the requests to appropriate internal methods and filling the inputStream with the result
 * 
 * It is possible to switch target-repository dynamically  (on every request). I.e. the client explicitly selects one of the repositories.
 *  
 * @author vronk
 *
 */
public class MDRepoProxy extends RepoProxy {

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("MDRepoProxy");
	
	private String proxy_key = "mdrepository";

	
	private final static HashMap<String,String> hash_sruoperations = new HashMap<String,String>();
	 static
	 {	 
		 hash_sruoperations.put("searchRetrieve", Query.RECORDSET);
		 hash_sruoperations.put("explain", Query.MODEL); /* could be terms*/ 
		 hash_sruoperations.put("scan", Query.VALUES);
	 }
	
	
	 public MDRepoProxy(Query q) {
			super(q);
	}

	/**
	 * Override of the basic method provided by the super-class 
	 */	
	@Override	
	public String getBaseURI() {		
		String uri = Repositories.getRepositories().getRepositoryPath(getSourceAction().getRepository());		
		return uri;
	}
	
	@Override
	public String getProxyKey() {
		return proxy_key;
	}
	
	/**
	 * A mapping between the actionkeys in the request and the operation-parameter expected by the MDRepository 
	 */
	private final static HashMap<String,String> urls = new HashMap<String,String>();
	 static
	 {	 urls.put("collection", "?operation=getCollections&collection=");
	  	 urls.put("model", "?operation=queryModel&q=");
	  	 urls.put("values", "?operation=scanIndex&q=");
	 	 urls.put("search", "?operation=searchRetrieve&query=");
	 	 urls.put("record", "?operation=searchRetrieve&query=");
	 } 
 
	 
	 // check and set defaults
	 @Override
	 public void checkParams() {
		// set defaults
		//	super.checkParams();
			/*
		if ( getParams().get("startRecord") == null){
				addParam("startRecord","1");
			}
		if ( getParams().get("maximumRecords") == null){
				addParam("maximumRecords","50");
			}
			
		if ( getParams().get("x-cmd-maxdepth") == null){
				if (getParams().get("maxdepth") == null){
					addParam("maxdepth","2");
				} 
		} else {
				addParam("maxdepth",getParams().get("x-cmd-maxdepth")[0]);
		}
			
		if ( getParams().get("x-cmd-repository") == null){
				if (getParams().get("repository") == null){
					addParam("repository",WorkspaceProfile.getRepositoryByIndex(0));
				} 
		} else {
				addParam("repository",getParams().get("x-cmd-repository")[0]);
		}
			
		if ( getParams().get("x-cmd-cache") == null){
				if (getParams().get("cache") == null){
					addParam("cache",Cache.SKIP);
				} 
		} else {
				addParam("cache",getParams().get("x-cmd-cache")[0]);
		}
			
		if ( getParams().get("x-cmd-lang") == null){
				if (getParams().get("lang") == null){
					addParam("lang","en");
				} 
		} else {
				addParam("lang",getParams().get("x-cmd-lang")[0]);
		}
			
		if ( getParams().get("x-cmd-format") == null){
			// TODO moved into BasicAction 
			
				if (getParams().get("format") == null){
					if (getSourceAction().getFormat() == null) {
						addParam("format","xml");
					} else if (getSourceAction().getFormat().equals("")){
						addParam("format","xml");
					} else {
						addParam("format",getSourceAction().getFormat());
					}
				}
				
		} else {
				addParam("format",getParams().get("x-cmd-format")[0]);
		}
		 
		//todo
		addParam("fullformat",getSourceAction().getFullFormat());
		//super.checkParams();
		*/
		// TODO ?????
		 if (getSourceAction().getActionkey().equals("sru") || getSourceAction().getActionkey().equals("fcs")){
			 getQuery().setType(hash_sruoperations.get(getParam("operation")));
			 getSourceAction().setActionkey(hash_sruoperations.get(getParam("operation")));
		 }
	 }
	 
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws CQLParseException 
	 * @throws Exception 
	 */
	 @Override
	public URL getTargetRequest() throws IOException, CQLParseException {
        
		//Query query = new Query(getParam("query"),getSourceAction().getActionkey());
	
		log.debug("MDRP.getQuery:" + getParam("query"));
		log.debug("MDRP.getActionkey:" + getSourceAction().getActionkey());		

	// check if the query could get parsed
		if (getQuery().isStatus(Query.PARSEERROR)) {
			log.debug("MDRP.query.PARSEERROR:" + getQuery().getMsg());
			throw new CQLParseException("MDRP.query.PARSEERROR:" + getQuery().getMsg());
			//getSourceAction().getDiagnostics().Add(Diagnostic.SYSTEM_ERROR, "MDRP.query.PARSEERROR:" + query.getMsg());
			//TODO exception ??
			//return null;
		} else {
			//query.setParams(getParams());
			//query.setMaxdepth(Integer.valueOf(getParam("maxdepth")));		
			//query.setCollection(getParam("collections"));
			//query.setColumns(getParam("columns"));
			//query.setMaximumItems(getParam("maximumRecords"));
			//query.setStartItem(getParam("startRecord"));
			//query.setOptions(getParam("options"));
			//query.setSort(getParam("sort"));
		
			URL targetURL = null;
			targetURL =new URL( getBaseURL(), urls.get(getSourceAction().getActionkey()) + getQuery().toURLParam());
		
			log.debug("MDRPA.targetURL:" + targetURL);
	        return targetURL;
		}
		 
	}
	
}